// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SceneComponentRegistry.hpp"

#include "component/display/Camera.hpp"
#include "component/display/material/SimpleTexturedMaterial.hpp"
#include "component/display/MeshRenderer.hpp"
#include "component/display/mesh/CubeMeshComponent.hpp"

#ifdef DAWN_ENABLE_PHYSICS
  #include "component/physics/CubeCollider.hpp"
  #include "component/physics/SphereCollider.hpp"
#endif

using namespace Dawn;

std::shared_ptr<SceneComponent> SceneComponentRegistry::createComponent(
  const std::string &type,
  const std::shared_ptr<SceneItem> &item
) {
  if(type.length() == 0) {
    assertUnreachable("Component type is empty!");

  } else if(type == "Camera") {
    return item->addComponent<Camera>();

  } else if(type == "SimpleTexturedMaterial") {
    return item->addComponent<SimpleTexturedMaterial>();

  } else if(type == "MeshRenderer") {
    return item->addComponent<MeshRenderer>();

  } else if(type == "CubeMesh" || type == "CubeMeshComponent") {
    return item->addComponent<CubeMeshComponent>();

  #ifdef DAWN_ENABLE_PHYSICS
    } else if(type == "CubeCollider") {
      return item->addComponent<CubeCollider>();

    } else if(type == "SphereCollider") {
      return item->addComponent<SphereCollider>();
  #endif

  } else {
    assertUnreachable("Unknown component type: %s", type.c_str());
  }

  return nullptr;
}